import re
from scipy.io import loadmat
import glob
import numpy as np

from neuro_helper.entity import TemplateName


def get_all_raw_files(tpt_name: TemplateName, task_name, scan_id):
    root_dir = "/group/northoff/share/mg/hcp/ready"
    if task_name == "Rest":
        task_name = "Restin"

    files = glob.glob(f"{root_dir}/*_MEG_{scan_id}-{task_name}_*[tr]megpreproc_rois-{tpt_name}.mat")
    files.sort()

    scans = {}
    for file in files:
        found = re.findall('_[0-9]+-', file)
        if not len(found) == 1:
            print("WARNING: Cannot find unique scan id in %s" % file)
            continue
        scan_id = int(found[0][1:-1])

        found = re.findall('[0-9]+_MEG', file)
        if not len(found) == 1:
            print("WARNING: Cannot find unique subject id in %s" % file)
            continue
        subj_id = found[0].replace("_MEG", "")

        if scan_id not in scans:
            scans[scan_id] = []
        scans[scan_id].append((subj_id, file))

    return scans


def load_raw_file(file):
    raw_data = loadmat(file)["data"]
    fs = raw_data["fsample"].item().item()
    data = np.concatenate(raw_data["trial"].item()[0], axis=1)

    return data, fs


def task_order(with_rest=True):
    if with_rest:
        out = ["Rest"]
    else:
        out = []
    return out + ["StoryM", "Motort", "Wrkmem"]


import os
from enum import Enum


class AssetCategory(Enum):
    HCP1200 = "hcp1200"


def get_full_path(category: AssetCategory, name: str):
    if not name:
        raise ValueError("file name is None.")
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), category.value, name)


import os
import cifti
import seaborn as sns

from neuro_helper.entity import TemplateName
from neuro_helper.colormap import cole_data, schaefer7_data

directory = "figures"
sns.set_style("whitegrid")
PC_colors = ["#637687", "#A20325"]
# PC_colors_tuple = [(99 / 256, 118 / 256, 135 / 256, 1), (162 / 255, 3 / 255, 37 / 255, 1)]
PC_colors_tuple = [(0 / 256, 79 / 256, 255 / 256, 1), (162 / 255, 3 / 255, 37 / 255, 1)]
PC_labels = ["Periphery", "Core"]
task_colors = ["#f032e6", "#aaffc3", "#637687"]
task_rest_colors = ["#B89B49"] + task_colors
PMC_labels = ["Periphery", "Intermediate", "Core"]
PMC_colors = ["#5975a4", "#cc8963", "#5f9e6e"]
PMC_colors_tuple = [(34 / 256, 45 / 256, 64 / 256, 1), (80 / 256, 54 / 256, 29 / 256, 1),
                    (38 / 256, 62 / 256, 44 / 256, 1)]
font_scale = 1.1
sns.set(font_scale=font_scale, style="whitegrid")
template_meta_combination = [
    (TemplateName.SCHAEFER_200_7, "pc"),
    (TemplateName.COLE_360, "pce"),
    (TemplateName.COLE_360, "pcr")
]

net_meta_C = {"pc": "C", "pce": "EC", "pcr": "RC"}


def savefig(fig, name, bbox_inches="tight", extra_artists=(), low=False, transparent=False):
    dpi = 80 if low else 600
    if not os.path.exists(directory):
        os.makedirs(directory)
    file_path = f"{directory}/{name}.png"
    fig.savefig(file_path, dpi=dpi, transparent=transparent, bbox_inches=bbox_inches, bbox_extra_artists=extra_artists)
    return os.getcwd() + os.sep + file_path


def savemap(name, data, brain_mask, *axes):
    if not os.path.exists(directory):
        os.makedirs(directory)
    file_path = f"{directory}/{name}.dtseries.nii"
    # noinspection PyTypeChecker
    cifti.write(file_path, data, axes + (brain_mask,))
    return os.getcwd() + os.sep + file_path


def make_net_palette(template_name: TemplateName):
    if template_name == TemplateName.COLE_360:
        return cole_data
    elif template_name == TemplateName.SCHAEFER_200_7:
        return schaefer7_data
    else:
        raise ValueError(f"{template_name} has no network template.")


def make_lh_pallete(palette):
    return [palette[0], ] * 7 + [palette[-1], ] * 5


def net_labels(tpt_name: TemplateName, two_line=True):
    if tpt_name == TemplateName.COLE_360:
        names = ['Visual1', 'Visual2', 'Auditory', 'Somatomotor', 'Dorsal\nAttention', 'Posterior\nMultimodal',
                 'Ventral\nMultimodal', 'Orbito\nAffective', 'Language', 'Cingulo\nOpercular', 'FPC', 'DMN']
    elif tpt_name == TemplateName.SCHAEFER_200_7:
        names = ['Visual', 'Somatomotor', 'Dorsal\nAttention', 'Salience', 'Limbic', 'FPC', 'DMN']
    else:
        raise ValueError(f"{tpt_name} not defined in net_labels")

    return names if two_line else [x.replace("\n", " ") for x in names]



from enum import Enum


class Space(Enum):
    K4 = "4k"
    K4_CORTEX = "4k_cortex"
    K32 = "32k"
    K32_CORTEX = "32k_cortex"
    K59 = "59k"
    K59_CORTEX = "59k_cortex"

    def __str__(self):
        return self.value

    @property
    def is_cortex(self):
        return "_cortex" in self.value


class TopoName(Enum):
    T1T2 = "t1t2"
    MARGULIES_GRADIENT = "gradient"
    ANT_POST_GRADIENT = "coord"
    MEDIAL_WALL = "medial_wall"

    def __str__(self):
        return self.value


class TemplateName(Enum):
    SCHAEFER_200_7 = "sh2007"
    SCHAEFER_200_17 = "sh20017"
    COLE_360 = "cole"
    WANG = "wang"

    def __str__(self):
        return self.value



import glob
import os
import cifti
import numpy as np

from neuro_helper.entity import Space


def get_all_raw_files(task_name, mode, id, space: Space):
    if space == Space.K32:
        pattern = f"/data/hcp/3T/#SUBJ#/MNINonLinear/Results/[tr]fMRI_{task_name}#SCAN#_[LR][LR]/*_Atlas_MSMAll_hp2000_clean.dtseries.nii"
    elif space == Space.K59:
        pattern = f"/data/hcp/7T/#SUBJ#/MNINonLinear/Results/[tr]fMRI_{task_name}#SCAN#_7T_*/*_7T_*_Atlas_1.6mm_MSMAll_hp2000_clean.dtseries.nii"
    else:
        raise ValueError(f"{space} doesn't have fMRI HCP raw files")

    if mode == "by_subj":
        pattern = pattern.replace("#SUBJ#", id).replace("#SCAN#", "*")
    elif mode == "by_scan":
        pattern = pattern.replace("#SUBJ#", "*").replace("#SCAN#", id)
    else:
        raise Exception("%s mode is not defined" % mode)

    files = glob.glob(pattern)
    files.sort()

    dictionary = {}
    for file in files:
        path_parts = file.split(os.sep)
        subj_id = path_parts[-5]
        scan_id = path_parts[-2].split("_")[1].replace(task_name, "")

        key = scan_id if mode == "by_scan" else "SUBJ"
        if key not in dictionary:
            dictionary[key] = {}
        if subj_id not in dictionary[key]:
            dictionary[key][subj_id] = []

        dictionary[key][subj_id].append((scan_id, file))

    return dictionary


def load_raw_file(file: str, space: Space):
    data = cifti.read(file)[0].T
    if space == Space.K59:
        fs = 1.0
    elif space == Space.K32:
        fs = 1000 / 720
    else:
        raise ValueError(f"{space} is not defined in loading a single raw file")
    return data, fs


def load_raw_files(input, space: Space, merge_func, **kwargs):
    if type(input) == str:
        data, fs = load_raw_file(input, space)
        return data, fs, input
    elif type(input) == list and len(input) == 1:
        data, fs = load_raw_file(input[0][1], space)
        return data, fs, input[0][1]
    elif type(input) == list:
        _, files = list(zip(*input))
        return merge_func(files, space, **kwargs)
    else:
        raise ValueError("input type is not valid")


def concat_scans(files, space: Space, **kwargs):
    cut_ratio = kwargs.get("cut_ratio")
    cut_sample = kwargs.get("cut_sample")
    output = []
    shared_fs = None

    for file in files:
        data, fs = load_raw_file(file, space)
        if shared_fs is None:
            shared_fs = fs
        elif not fs == shared_fs:
            raise Exception("Incompatible FS, cannot merge scans in %s" % files)

        if cut_ratio is not None:
            data_cut_sample = int(data.shape[1] * cut_ratio)
        elif cut_sample is not None:
            data_cut_sample = cut_sample
        else:
            data_cut_sample = data.shape[1]

        if data_cut_sample > data.shape[1]:
            raise Exception("Large cut sample in merging %s" % file)
        output.append(data[:, :data_cut_sample])

    return np.column_stack(output), shared_fs, "CONCAT " + ", ".join(files)


def average_scans(files, space: Space, **kwargs):
    output = []
    shared_fs = None

    min_length = np.Inf
    for file in files:
        data, fs = load_raw_file(file, space)
        if shared_fs is None:
            shared_fs = fs
        elif not fs == shared_fs:
            print(f"Shared Fs: {shared_fs} and Fs: {fs}")
            raise Exception(f"Incompatible FS, cannot merge scans in {', '.join(files)}")

        min_length = min(min_length, data.shape[1])
        output.append(data)

    output = np.asarray([x[:, :min_length] for x in output])
    return output.mean(axis=0), shared_fs, "AVERAGE " + ", ".join(files)






import numpy as np
import cifti
from pandas import DataFrame, Series
from neuro_helper.entity import TopoName, Space, TemplateName
from neuro_helper.assets import manager

_loaded_templates = {}

file_names = {
    TopoName.MEDIAL_WALL: {
        Space.K32: None,
        Space.K32_CORTEX: "topo.medial_wall.32k_cortex.dlabel.nii",
        Space.K59: "topo.medial_wall.59k.dlabel.nii",
        Space.K59_CORTEX: "topo.medial_wall.59k_cortex.dlabel.nii"
    },
    TopoName.T1T2: {
        Space.K32: None,
        Space.K32_CORTEX: "topo.t1t2.32k_cortex.dscalar.nii",
        Space.K59: None,
        Space.K59_CORTEX: "topo.t1t2.59k_cortex.dscalar.nii",
    },
    TopoName.ANT_POST_GRADIENT: {
        Space.K32: "topo.coord.32k.dscalar.nii",
        Space.K32_CORTEX: "topo.coord.32k_cortex.dscalar.nii",
        Space.K59: None,
        Space.K59_CORTEX: None,
    },
    TopoName.MARGULIES_GRADIENT: {
        Space.K32: "topo.margulies2016.32k.dscalar.nii",
        Space.K32_CORTEX: "topo.margulies2016.32k_cortex.dscalar.nii",
        Space.K59: "topo.margulies2016.59k.dscalar.nii",
        Space.K59_CORTEX: "topo.margulies2016.59k_cortex.dscalar.nii",
    },
    TemplateName.SCHAEFER_200_7: {
        Space.K32: None,
        Space.K32_CORTEX: "template.schaefer2018.2007.32k_cortex.dlabel.nii",
        Space.K59: None,
        Space.K59_CORTEX: "template.schaefer2018.2007.59k_cortex.dlabel.nii",
    },
    TemplateName.SCHAEFER_200_17: {
        Space.K32: None,
        Space.K32_CORTEX: "template.schaefer2018.20017.32k_cortex.dlabel.nii",
        Space.K59: None,
        Space.K59_CORTEX: None,
    },
    TemplateName.COLE_360: {
        Space.K4: None,
        Space.K4_CORTEX: "template.cole.36012.4k_cortex.dlabel.nii",
        Space.K32: "template.cole.36012.32k.dlabel.nii",
        Space.K32_CORTEX: "template.cole.36012.32k_cortex.dlabel.nii",
        Space.K59: "template.cole.36012.59k.dlabel.nii",
        Space.K59_CORTEX: "template.cole.36012.59k_cortex.dlabel.nii"
    },
    TemplateName.WANG: {
        Space.K32: "template.wang2015.32k.dlabel.nii",
        Space.K32_CORTEX: "template.wang2015.32k_cortex.dlabel.nii",
        Space.K59: "template.wang2015.59k.dlabel.nii",
        Space.K59_CORTEX: "template.wang2015.59k_cortex.dlabel.nii"
    }
}


def get_full_path(file_name):
    return manager.get_full_path(manager.AssetCategory.HCP1200, file_name)


def _get_or_load(key, loaded):
    if key not in _loaded_templates:
        if callable(loaded):
            _loaded_templates[key] = loaded()
        else:
            _loaded_templates[key] = loaded
    return _loaded_templates[key]


def get_topo_dataframe(topo_name: TopoName, template_name: TemplateName, space: Space):
    if topo_name == TopoName.T1T2:
        return _get_t1t2_topo(template_name, space)
    elif topo_name == TopoName.MARGULIES_GRADIENT:
        return _get_gradient_topo(template_name, space)
    elif topo_name == TopoName.ANT_POST_GRADIENT:
        return _get_coordinates_topo(template_name, space)
    elif topo_name == TopoName.MEDIAL_WALL:
        return _get_medial_wall_topo(space)
    else:
        raise ValueError(f"{topo_name} is not defined")


def _get_medial_wall_topo(space: Space):
    return _get_or_load(f"{TopoName.MEDIAL_WALL}:{space}",
                        lambda: cifti.read(get_full_path(file_names[TopoName.MEDIAL_WALL][space]))[0].squeeze())


def _get_t1t2_topo(template_name: TemplateName, space: Space):
    def load():
        voxels = cifti.read(get_full_path(file_names[TopoName.T1T2][space]))[0].squeeze()
        mask, _, networks, regions, _ = get_template(template_name, space)
        mask_no_wall = mask[_get_medial_wall_topo(space) == 0]
        topo = DataFrame({"region": Series(dtype=str), "network": Series(dtype=str), "t1t2": Series(dtype=float)})
        for i, (reg, net) in enumerate(zip(regions, networks)):
            topo.loc[i, :] = reg, net, voxels[mask_no_wall == i + 1].mean()
        return topo

    return _get_or_load(f"{TopoName.T1T2}:{template_name}:{space}", load)


def _get_gradient_topo(template_name: TemplateName, space: Space):
    def load():
        mask, _, networks, regions, _ = get_template(template_name, space)
        voxels = cifti.read(get_full_path(file_names[TopoName.MARGULIES_GRADIENT][space]))[0].squeeze()
        mask_no_wall = mask[_get_medial_wall_topo(space) == 0]
        topo = DataFrame({"region": Series(dtype=str), "network": Series(dtype=str), "gradient": Series(dtype=float)})
        for i, (reg, net) in enumerate(zip(regions, networks)):
            topo.loc[i, :] = reg, net, voxels[mask_no_wall == i + 1].mean()
        return topo

    return _get_or_load(f"{TopoName.MARGULIES_GRADIENT}:{template_name}:{space}", load)


def _get_coordinates_topo(template_name: TemplateName, space: Space):
    def load():
        mask, _, networks, regions, _ = get_template(template_name, space)
        voxels = cifti.read(get_full_path(file_names[TopoName.ANT_POST_GRADIENT][space]))[0].T
        mask_no_wall = mask[_get_medial_wall_topo(space) == 0]
        topo = DataFrame({"region": Series(dtype=str), "network": Series(dtype=str),
                          "coord_x": Series(dtype=float), "coord_y": Series(dtype=float),
                          "coord_z": Series(dtype=float)})
        for i, (reg, net) in enumerate(zip(regions, networks)):
            x, y, z = voxels[mask_no_wall == i + 1, :].mean(axis=0)
            topo.loc[i, :] = reg, net, x, y, z
        return topo

    return _get_or_load(f"{TopoName.ANT_POST_GRADIENT}:{template_name}:{space}", load)


def parcellate(tpt_name: TemplateName, space: Space, voxels):
    mask, _, networks, regions, _ = get_template(tpt_name, space)

    if voxels.shape[0] > mask.size:
        print("INFO: mask is smaller than data. Probably sub-cortex is in the data."
              " Padding mask with 0s (assuming bg=0)")
        mask = np.append(mask, np.zeros(voxels.shape[0] - mask.size))
    elif voxels.shape[0] < mask.size:
        raise ValueError("Mask size is smaller than data size. Something is fishy here!")

    output = np.zeros((len(regions), voxels.shape[1]))
    for ri in range(len(regions)):
        output[ri] = voxels[mask == ri + 1].mean()
    return output


def get_template(name: TemplateName, space: Space):
    return _loaded_templates.get(f"{name}:{space}")


def load_schaefer_template(space: Space, reg_count, net_count):
    if reg_count == 200 and net_count == 7:
        tpt_name = TemplateName.SCHAEFER_200_7
    elif reg_count == 200 and net_count == 17:
        tpt_name = TemplateName.SCHAEFER_200_17
    else:
        raise ValueError(f"SCHAEFER with {reg_count} regions and {net_count} networks is not defined")

    name = f"{tpt_name}:{space}"
    if name not in _loaded_templates:
        mask, (lbl_axis, brain_axis) = \
            cifti.read(get_full_path(file_names[tpt_name][space]))
        mask = np.squeeze(mask)
        lbl_dict = lbl_axis.label.item()
        regions = np.asarray([lbl_dict[key][0] for key in list(lbl_dict.keys())])[1:]
        networks = [x.split("_")[2] for x in regions]
        unique_networks = np.unique(networks)
        _loaded_templates[name] = mask, unique_networks, networks, regions, brain_axis
    return name


def load_cole_template(space: Space):
    name = f"{TemplateName.COLE_360}:{space}"
    if name not in _loaded_templates:
        mask, (lbl_axis, brain_axis) = cifti.read(get_full_path(file_names[TemplateName.COLE_360][space]))
        mask = np.squeeze(mask)
        lbl_dict = lbl_axis.label.item()
        regions = np.asarray([lbl_dict[x][0] for x in np.unique(mask)])[1:]
        networks = ["".join(x.split("_")[0].split("-")[:-1]) for x in regions]
        unique_networks = np.unique(networks)
        _loaded_templates[name] = mask, unique_networks, networks, regions, brain_axis
    return name


def load_wang_template(space: Space):
    name = f"{TemplateName.WANG}:{space}"
    if name not in _loaded_templates:
        mask, (lbl_axis, brain_axis) = cifti.read(get_full_path(file_names[TemplateName.COLE_360][space]))
        mask = np.squeeze(mask)
        lbl_dict = lbl_axis.label.item()
        regions = np.asarray([lbl_dict[x][0] for x in np.unique(mask)])[1:]
        networks = ["".join(x.split("_")[0].split("-")[:-1]) for x in regions]
        unique_networks = np.unique(networks)
        _loaded_templates[name] = mask, unique_networks, networks, regions, brain_axis
    return name


def get_net(net_lbl, template_name: TemplateName):
    if template_name == TemplateName.COLE_360:
        if net_lbl == "pce":
            return {"P": ["Visual1", "Visual2", "Auditory", "Somatomotor"],
                    "EC": ["DorsalAttention", "PosteriorMultimodal", "VentralMultimodal", "OrbitoAffective",
                           "Language", "CinguloOpercular", "Frontoparietal", "Default"]}
        elif net_lbl == "pcr":
            return {"P": ["Visual1", "Visual2", "Auditory", "Somatomotor"],
                    "RC": ["CinguloOpercular", "Frontoparietal", "Default"]}
    elif template_name == TemplateName.SCHAEFER_200_7:
        if net_lbl == "pc":
            return {"P": ['Vis', 'SomMot', 'DorsAttn', 'SalVentAttn'],
                    "C": ['Limbic', 'Cont', 'Default']}

    raise ValueError(f"{template_name} and {net_lbl} is not defined")


def net_order(template_name: TemplateName):
    if template_name == TemplateName.COLE_360:
        return ["Visual1", "Visual2", "Auditory", "Somatomotor", "DorsalAttention", "PosteriorMultimodal",
                "VentralMultimodal", "OrbitoAffective", "Language", "CinguloOpercular", "Frontoparietal", "Default"]
    elif template_name == TemplateName.SCHAEFER_200_7:
        return ['Vis', 'SomMot', 'DorsAttn', 'SalVentAttn', 'Limbic', 'Cont', 'Default']

    raise Exception(f"{template_name} not defined")


def net_labels(tpt_name: TemplateName, two_line=True):
    if tpt_name == TemplateName.COLE_360:
        names = ['Visual1', 'Visual2', 'Auditory', 'Somatomotor', 'Dorsal\nAttention', 'Posterior\nMultimodal',
                 'Ventral\nMultimodal', 'Orbito\nAffective', 'Language', 'Cingulo\nOpercular', 'FPC', 'DMN']
    elif tpt_name == TemplateName.SCHAEFER_200_7:
        names = ['Visual', 'Somatomotor', 'Dorsal\nAttention', 'Salience', 'Limbic', 'FPC', 'DMN']
    else:
        raise ValueError(f"{tpt_name} not defined in net_labels")

    return names if two_line else [x.replace("\n", " ") for x in names]



import numpy as np
from scipy import signal, stats


def fir_filter_data(data, fs, order=None, min_cycles=4, max_freq_low=None, min_freq_high=None, pass_type="bp"):
    n_channel, n_sample = data.shape
    total_time = n_sample / fs

    if order is None:
        if n_sample > 800:
            order = 201
        elif 250 < n_sample < 350:
            order = 51
        else:
            raise Exception("Undefined number of points (%s) for filtering" % n_sample)

    freq_l = np.ceil(min_cycles / total_time * 1000) / 1000
    freq_h = np.floor(fs / 2 * 1000) / 1000

    if max_freq_low:
        freq_l = np.max([freq_l, max_freq_low])
    if min_freq_high:
        freq_h = np.min([freq_h, min_freq_high])

    if pass_type == "bp":
        pass_zero = False
        freq = [freq_l, freq_h]
    elif pass_type == "lp":
        pass_zero = True
        freq = freq_h
        freq_l = None
    elif pass_type == "hp":
        pass_zero = False
        freq = freq_l
        freq_h = None
    else:
        raise Exception("pass_type not defined")

    # noinspection PyTypeChecker
    return signal.filtfilt(signal.firwin(order, freq, window='hanning', pass_zero=pass_zero, fs=fs),
                           [1], data), freq_l, freq_h


def welch_psd(data, freq_l, fs):
    n_sample = data.shape[1]
    n_fft = 2 ** np.ceil(np.log2(n_sample))
    win_size = int(2 * 1.6 / freq_l * fs)

    if win_size > n_sample:
        return None, None

    return signal.welch(data, fs=fs, nfft=n_fft, detrend=None,
                        window="hanning", nperseg=win_size, noverlap=int(0.9 * win_size))


def percent_change(base_cond, new_cond):
    increase = new_cond - base_cond
    return increase / base_cond * 100


def anova_table(aov):
    aov['mean_sq'] = aov[:]['sum_sq'] / aov[:]['df']

    aov['eta_sq'] = aov[:-1]['sum_sq'] / sum(aov['sum_sq'])

    aov['omega_sq'] = (aov[:-1]['sum_sq'] - (aov[:-1]['df'] * aov['mean_sq'][-1])) / (
                sum(aov['sum_sq']) + aov['mean_sq'][-1])

    cols = ['sum_sq', 'df', 'mean_sq', 'F', 'PR(>F)', 'eta_sq', 'omega_sq']
    aov = aov[cols]
    return aov


def cohend(d1, d2):
    n1, n2 = len(d1), len(d2)
    s1, s2 = np.var(d1, ddof=1), np.var(d2, ddof=1)
    s = np.sqrt(((n1 - 1) * s1 + (n2 - 1) * s2) / (n1 + n2 - 2))
    u1, u2 = np.mean(d1), np.mean(d2)
    return (u1 - u2) / s


def correlate_rows(data):
    n_items = data.shape[0]
    corr = np.zeros((n_items, n_items))
    for ri in range(n_items):
        for rj in range(ri, n_items):
            corr_val, _ = stats.pearsonr(data[ri, :], data[rj, :])
            corr[ri, rj] = corr[rj, ri] = corr_val

    return corr


def icc(y, icc_type='icc2'):
    """ Calculate intraclass correlation coefficient for data within
        Brain_Data class
    icc Formulas are based on:
    Shrout, P. E., & Fleiss, J. L. (1979). Intraclass correlations: uses in
    assessing rater reliability. Psychological bulletin, 86(2), 420.
    Code from https://github.com/cosanlab/nltools/blob/master/nltools/data/brain_data.py
    Args:
        y: subjects x sessions
        icc_type: type of icc to calculate (icc: voxel random effect,
                icc2: voxel and column random effect, icc3: voxel and
                column fixed effect)
    Returns:
        icc: (np.array) intraclass correlation coefficient
    """
    # n, k
    [n_subjects, n_scans] = y.shape

    # Degrees of Freedom
    dfc = n_scans - 1
    dfe = (n_subjects - 1) * dfc
    dfr = n_subjects - 1

    # Sum Square Total
    mean_y = np.mean(y)
    SST = ((y - mean_y) ** 2).sum()

    # create the design matrix for the different levels
    x = np.kron(np.eye(n_scans), np.ones((n_subjects, 1)))  # sessions
    x0 = np.tile(np.eye(n_subjects), (n_scans, 1))  # subjects
    X = np.hstack([x, x0])

    # Sum Square Error
    predicted_y = np.dot(np.dot(np.dot(X, np.linalg.pinv(np.dot(X.T, X))),
                                X.T), y.flatten('F'))
    residuals = y.flatten('F') - predicted_y
    sse = (residuals ** 2).sum()

    mse = sse / dfe

    # Sum square column effect - between colums
    ssc = ((np.mean(y, 0) - mean_y) ** 2).sum() * n_subjects
    msc = ssc / dfc / n_subjects

    # Sum Square subject effect - between rows/subjects
    ssr = SST - ssc - sse
    msr = ssr / dfr

    if icc_type == 'icc1':
        # icc(2,1) = (mean square subject - mean square error) /
        # (mean square subject + (k-1)*mean square error +
        # k*(mean square columns - mean square error)/n)
        # icc = (msr - MSRW) / (msr + (k-1) * MSRW)
        raise NotImplementedError("This method isn't implemented yet.")
    elif icc_type == 'icc2':
        # icc(2,1) = (mean square subject - mean square error) /
        # (mean square subject + (k-1)*mean square error +
        # k*(mean square columns - mean square error)/n)
        icc_value = (msr - mse) / (msr + (n_scans - 1) * mse + n_scans * (msc - mse) / n_subjects)
    elif icc_type == 'icc3':
        # icc(3,1) = (mean square subject - mean square error) /
        # (mean square subject + (k-1)*mean square error)
        icc_value = (msr - mse) / (msr + (n_scans - 1) * mse)
    else:
        raise ValueError(f"{icc_type} is not defined.")

    return icc_value
